@@ -22,29 +22,34 @@ Functions
 
 __date__ = "2021-07-02"
 
-import typing
-import os
 import hashlib
-import urllib.parse
 import logging
+import os
+import typing
+
 import yaml
 
-import fair.registry.requests as fdp_req
-import fair.exceptions as fdp_exc
 import fair.configuration as fdp_conf
+import fair.exceptions as fdp_exc
 import fair.identifiers as fdp_id
 import fair.registry.file_types as fdp_file
+import fair.registry.requests as fdp_req
 import fair.registry.versioning as fdp_ver
 
-def get_write_storage(uri: str, cfg: typing.Dict) -> str:
+logger = logging.getLogger("FAIRDataPipeline.Storage")
+
+
+def get_write_storage(uri: str, write_data_store: str, token: str) -> str:
     """Construct storage root if it does not exist
 
     Parameters
     ----------
     uri : str
         end point of the RestAPI
-    str : typing.Dict
-        path of the config file
+    write_data_store : str
+        path of the write data store
+    token: str
+        registry access token
 
     Returns
     -------
@@ -56,34 +61,58 @@ def get_write_storage(uri: str, cfg: typing.Dict) -> str:
     fdp_exc.UserConfigError
         If 'write_data_store' not present in the working config or global config
     """
-
-    _write_data_store = fdp_conf.write_data_store(cfg)
+    logger.debug("Constructing a storage root for '%s'", write_data_store)
 
     # Convert local file path to a valid data store path
-    _write_store_root = f"file://{_write_data_store}"
+    _write_store_root = f"file://{write_data_store}"
     if _write_store_root[-1] != os.path.sep:
         _write_store_root += os.path.sep
 
-    # Check if the data store already exists by querying for it
-    _search_root = fdp_req.get(
-        uri, "storage_root", params={"root": _write_store_root}
-    )
-
-    # If the data store already exists just return the URI else create it
-    # and then do the same
-    if not _search_root:
-        _post_data = {"root": _write_store_root, "local": True}
-        _storage_root = fdp_req.post(uri, "storage_root", data=_post_data)
-        return _storage_root["url"]
-    else:
+    if _search_root := fdp_req.get(
+        uri, "storage_root", token, params={"root": _write_store_root}
+    ):
         return _search_root[0]["url"]
 
+    _post_data = {"root": _write_store_root, "local": True}
+    _storage_root = fdp_req.post(uri, "storage_root", token, data=_post_data)
+    return _storage_root["url"]
 
-def store_user(repo_dir: str, uri: str) -> str:
+
+def store_author(
+    uri: str, token: str, name: str, identifier: str = None, uuid: str = None
+) -> str:
+    """Creates an Author entry if one does not exist
+
+    Parameters
+    ----------
+    uri : str
+        registry RestAPI endpoint
+    token: str
+        registry access token
+    data: typing.Dict
+        author data to post
+    params: typing.Dict, optional
+        parameters to search if exists already
+
+    Returns
+    -------
+    str
+        URI for created author
+    """
+    _data = {"name": name, "identifier": identifier, "uuid": uuid}
+
+    return fdp_req.post_else_get(uri, "author", token, _data, {"name": name})
+
+
+def store_user(repo_dir: str, uri: str, token: str) -> str:
     """Creates an Author entry for the user if one does not exist
 
     Parameters
     ----------
+    repo_dir: str
+        repository directory
+    token: str
+        registry access token
     uri : str
         registry RestAPI endpoint
 
@@ -92,45 +121,44 @@ def store_user(repo_dir: str, uri: str) -> str:
     str
         URI for created author
     """
+
     _user = fdp_conf.get_current_user_name(repo_dir)
-    _data = {}
-    if len(_user) > 1:
-        _data['name'] = ' '.join(_user)
-    else:
-        _data['name'] = _user[0]
+    name = " ".join(_user) if _user[1] else _user[0]
+    _id = None
+    _uuid = None
+
+    logger.debug("Storing user '%s'", name)
 
     try:
-        _orcid = fdp_conf.get_current_user_orcid(repo_dir)
-        _orcid = urllib.parse.urljoin(fdp_id.ORCID_URL, _orcid)
-        _data["identifier"] = _orcid
-        return fdp_req.post_else_get(
-            uri, "author", data=_data, params={"identifier": _orcid}
-        )
+        _id = fdp_conf.get_current_user_uri(repo_dir)
     except fdp_exc.CLIConfigurationError:
         _uuid = fdp_conf.get_current_user_uuid(repo_dir)
-        _data['uuid'] = _uuid
-        return fdp_req.post_else_get(
-            uri, "author", data=_data, params={"uuid": _uuid}
-        )
 
-def populate_file_type(uri:str):
+    return store_author(uri, token, name, _id, _uuid)
+
+
+def populate_file_type(uri: str, token: str) -> typing.List[typing.Dict]:
     """Populates file_type table with common file file_types
 
     Parameters
     ----------
     uri: str
         registry RestAPI end point
+    token: str
+        registry access token
     """
+    logger.debug("Adding file types to storage")
 
-    for _extension, _name in fdp_file.FILE_TYPES.items():
+    _type_objs = []
+
+    for _extension in fdp_file.FILE_TYPES:
         # Use post_else_get in case some file types exist already
-        fdp_req.post_else_get(
-            uri, "file_type",
-            data={"name": _name, "extension": _extension.lower()},
-            params={"extension": _extension.lower()}
-        )
+        _result = create_file_type(uri, _extension, token)
+        _type_objs.append(_result)
+    return _type_objs
 
-def create_file_type(uri: str, extension: str) -> str:
+
+def create_file_type(uri: str, extension: str, token: str) -> str:
     """Creates a new file type on the registry
 
     Parameters
@@ -139,6 +167,8 @@ def create_file_type(uri: str, extension: str) -> str:
         registry RestAPI end point
     extension : str
         file extension
+    token: str
+        registry access string
 
     Returns
     -------
@@ -146,14 +176,21 @@ def create_file_type(uri: str, extension: str) -> str:
         URI for created file type
     """
     _name = fdp_file.FILE_TYPES[extension]
+
+    logger.debug("Adding file type '%s' with extension '%s'", _name, extension)
+
     return fdp_req.post_else_get(
-        uri, "file_type",
+        uri,
+        "file_type",
+        token,
         data={"name": _name, "extension": extension.lower()},
-        params={"extension": extension.lower()}
+        params={"extension": extension.lower()},
     )
 
 
-def store_working_config(repo_dir: str, uri: str, work_cfg_yml: str) -> str:
+def store_working_config(
+    repo_dir: str, uri: str, work_cfg_yml: str, token: str
+) -> str:
     """Construct a storage location and object for the working config
 
     Parameters
@@ -164,6 +201,8 @@ def store_working_config(repo_dir: str, uri: str, work_cfg_yml: str) -> str:
         RestAPI end point
     work_cfg_yml : str
         location of working config yaml
+    token: str
+        registry access token
 
     Returns
     -------
@@ -175,7 +214,9 @@ def store_working_config(repo_dir: str, uri: str, work_cfg_yml: str) -> str:
     fair.exceptions.RegistryAPICallError
         if bad status code returned from the registry
     """
-    _root_store = get_write_storage(uri, work_cfg_yml)
+    logger.debug("Storing working config on registry")
+
+    _root_store = get_write_storage(uri, work_cfg_yml, token)
 
     _work_cfg = yaml.safe_load(open(work_cfg_yml))
     _work_cfg_data_store = _work_cfg["run_metadata"]["write_data_store"]
@@ -192,30 +233,26 @@ def store_working_config(repo_dir: str, uri: str, work_cfg_yml: str) -> str:
     _storage_loc_data = {
         "path": _rel_path,
         "storage_root": _root_store,
-        "public": _work_cfg['run_metadata']['public'],
+        "public": _work_cfg["run_metadata"].get("public", True),
         "hash": _hash,
     }
 
     try:
         _post_store_loc = fdp_req.post(
-            uri,
-            "storage_location",
-            data=_storage_loc_data
+            uri, "storage_location", token, data=_storage_loc_data
         )
     except fdp_exc.RegistryAPICallError as e:
-        if not e.error_code == 409:
+        if e.error_code != 409:
             raise e
         else:
             raise fdp_exc.RegistryAPICallError(
-                f"Cannot post storage_location "
-                f"'{_rel_path}' with hash"
-                f" '{_hash}', object already exists",
-                error_code=409
-            )
+                f"Cannot post storage_location '{_rel_path}' with hash '{_hash}', object already exists",
+                error_code=409,
+            ) from e
 
-    _user = store_user(repo_dir, uri)
+    _user = store_user(repo_dir, uri, token)
 
-    _yaml_type = create_file_type(uri, "yaml")
+    _yaml_type = create_file_type(uri, "yaml", token)
 
     _desc = f"Working configuration file for timestamp {_time_stamp_dir}"
     _object_data = {
@@ -226,16 +263,13 @@ def store_working_config(repo_dir: str, uri: str, work_cfg_yml: str) -> str:
     }
 
     return fdp_req.post_else_get(
-        uri, "object", data=_object_data, params={"description": _desc}
+        uri, "object", token, data=_object_data, params={"description": _desc}
     )
 
 
 def store_working_script(
-    repo_dir: str,
-    uri: str,
-    script_path: str,
-    working_config: str
-    ) -> str:
+    repo_dir: str, uri: str, script_path: str, working_config: str, token: str
+) -> str:
     """Construct a storage location and object for the CLI run script
 
     Parameters
@@ -248,6 +282,8 @@ def store_working_script(
         location of working CLI run script
     data_store : str
         data store path
+    token: str
+        registry access token
 
     Returns
     -------
@@ -259,8 +295,10 @@ def store_working_script(
     fair.exceptions.RegistryAPICallError
         if bad status code returned from the registry
     """
+    logger.debug("Storing working script on registry")
+
     _work_cfg = yaml.safe_load(open(working_config))
-    _root_store = get_write_storage(uri, working_config)
+    _root_store = get_write_storage(uri, working_config, token)
     _data_store = _work_cfg["run_metadata"]["write_data_store"]
 
     _rel_path = os.path.relpath(script_path, _data_store)
@@ -277,32 +315,28 @@ def store_working_script(
     _storage_loc_data = {
         "path": _rel_path,
         "storage_root": _root_store,
-        "public": _work_cfg['run_metadata']['public'],
+        "public": _work_cfg["run_metadata"].get("public", True),
         "hash": _hash,
     }
 
     try:
         _post_store_loc = fdp_req.post(
-            uri,
-            "storage_location",
-            data=_storage_loc_data
+            uri, "storage_location", token, data=_storage_loc_data
         )
     except fdp_exc.RegistryAPICallError as e:
-        if not e.error_code == 409:
+        if e.error_code != 409:
             raise e
         else:
             raise fdp_exc.RegistryAPICallError(
                 f"Cannot post storage_location "
                 f"'{_rel_path}' with hash"
                 f" '{_hash}', object already exists",
-                error_code=409
-            )
+                error_code=409,
+            ) from e
 
-    _user = store_user(repo_dir, uri)
+    _user = store_user(repo_dir, uri, token)
 
-    _shell_script_type = create_file_type(
-        uri, "sh"
-    )
+    _shell_script_type = create_file_type(uri, "sh", token)
 
     _time_stamp_dir = os.path.basename(os.path.dirname(script_path))
     _desc = f"Run script for timestamp {_time_stamp_dir}"
@@ -314,17 +348,14 @@ def store_working_script(
     }
 
     return fdp_req.post_else_get(
-        uri, "object", data=_object_data, params={"description": _desc}
+        uri, "object", token, data=_object_data, params={"description": _desc}
     )
 
 
 def store_namespace(
-    uri: str,
-    namespace_label: str,
-    full_name: str = None,
-    website: str = None
+    uri: str, token: str, name: str, full_name: str = None, website: str = None
 ) -> str:
-    """Create a namespace on the
+    """Create a namespace on the registry
 
     Parameters
     ----------
@@ -332,6 +363,8 @@ def store_namespace(
         endpoint of the registry
     namespace_label : str
         name of the Namespace
+    token : str
+        registry access token
     full_name : str, optional
         full title of the namespace, by default None
     website : str, optional
@@ -342,247 +375,162 @@ def store_namespace(
     str
         URL of the created namespace
     """
+    logger.debug("Storing namespace '%s' on registry", name)
+
     _data = {
-        "name": namespace_label,
+        "name": name,
         "full_name": full_name,
-        "website": website
+        "website": website,
     }
+
     return fdp_req.post_else_get(
-        uri, "namespace", data=_data, params={"name": namespace_label}
+        uri, "namespace", token, data=_data, params={"name": name}
     )
 
 
-def update_namespaces(cfg: typing.Dict) -> None:
-    """Ensure that the namespaces in the user config are created
-    """
-
-    _local_uri = fdp_conf.registry_url("local", cfg)
-
-    _namespaces = []
-    _new_block = []
-
-    if 'register' in cfg:
-        _block_cfg = cfg['register']
-
-        for item in _block_cfg:
-            if 'external_object' in item or 'data_product' in item:
-                if 'namespace_name' in item:
-                    _namespaces.append({
-                        'name': item['namespace_name'],
-                        'full_name': item.get('namespace_full_name', None),
-                        'website': item.get('namespace_website', None)
-                    })
-                elif 'namespace' in item and isinstance(item['namespace'], dict):
-                    _namespaces.append(item['namespace'])
-                _new_block.append(item)
-            elif 'namespace' in item:
-                _namespaces.append({
-                    'name': item['namespace'],
-                    'full_name': item.get('full_name', None),
-                    'website': item.get('website', None)
-                })
-            else:
-                _new_block.append(item)
-
-        for namespace in _namespaces:
-            store_namespace(
-                _local_uri,
-                namespace['name'],
-                namespace['full_name'],
-                namespace['website']
-            )
-        
-    store_namespace(
-        _local_uri,
-        fdp_conf.input_namespace(cfg)
-    )
-
-    store_namespace(
-        _local_uri,
-        fdp_conf.output_namespace(cfg)
-    )
-
-    if 'register' in cfg:
-        for item in _new_block:
-            if ('external_object' in item or 'data_product' in item) and 'namespace' in item:
-                if isinstance(item['namespace'], str):
-                    item['namespace_name'] = item['namespace']
-                elif isinstance(item['namespace'], dict):
-                    item['namespace_name'] = item['namespace']['name']
-                item['namespace_full_name'] = None
-                item['namespace_website'] = None
-                del item['namespace']
-        cfg['register'] = _new_block
-
-
 def store_data_file(
     uri: str,
     repo_dir: str,
+    token: str,
     data: typing.Dict,
     local_file: str,
-    cfg: typing.Dict,
-    public: bool
+    write_data_store: str,
+    public: bool,
 ) -> None:
 
-    _root_store = get_write_storage(uri, cfg)
-    _data_store = fdp_conf.write_data_store(cfg)
+    logger.debug("Storing data file '%s' on registry", local_file)
 
-    _rel_path = os.path.relpath(local_file, _data_store)
+    _root_store = get_write_storage(uri, write_data_store, token)
 
-    if 'version' not in data['use']:
+    _rel_path = os.path.relpath(local_file, write_data_store)
+
+    if "version" not in data["use"]:
         raise fdp_exc.InternalError(
             f"Expected version number for '{local_file}' "
             "registry submission but none found"
         )
 
-    _hash = calculate_file_hash(local_file)
-
-    _storage_loc_data = {
-        "path": _rel_path,
-        "storage_root": _root_store,
-        "public": public,
-        "hash": _hash,
-    }
-
-    _search_data = {
-        'hash': _hash
-    }
-
-    _post_store_loc = fdp_req.post_else_get(
-        uri,
-        "storage_location",
-        data=_storage_loc_data,
-        params=_search_data
+    _post_store_loc = _get_url_from_storage_loc(
+        local_file=local_file,
+        registry_uri=uri,
+        registry_token=token,
+        relative_path=_rel_path,
+        root_store_url=_root_store,
+        is_public=public,
     )
 
-    _user = store_user(repo_dir, uri)
+    _user = store_user(repo_dir, uri, token)
 
-    if 'file_type' in data:
-        _file_type = data['file_type']
-    else:
-        _file_type = os.path.splitext(local_file)[1]
-
-    _file_type = create_file_type(
-        uri, _file_type
+    _file_type = _get_url_from_file_type(
+        data=data,
+        local_file=local_file,
+        registry_uri=uri,
+        registry_token=token,
     )
 
-    # Namespace is read from the source information
-    if 'namespace_name' not in data:
-        raise fdp_exc.UserConfigError(
-            f"Expected 'namespace_name' for item '{local_file}'"
-        )
+    _namespace_url = _get_url_from_namespace(
+        data=data, label=local_file, registry_uri=uri, registry_token=token
+    )
 
-    _namespace_args = {
-        "uri": uri,
-        "namespace_label": data['namespace_name'],
-        "full_name": data['namespace_full_name']
-            if 'namespace_full_name' in data else None,
-        "website": data['namespace_website']
-            if 'namespace_website' in data else None
-    }
+    _obj_url = _get_url_from_object(
+        data=data,
+        registry_uri=uri,
+        registry_token=token,
+        user=_user,
+        storage_loc_url=_post_store_loc,
+        file_type_url=_file_type,
+    )
 
-    _namespace_url = store_namespace(**_namespace_args)
-
-    _desc = data['description'] if 'description' in data else None
-
-    _object_data = {
-        "description": _desc,
-        "file_type": _file_type,
-        "storage_location": _post_store_loc,
-        "authors": [_user]
-    }
-
-    try:
-        _obj_url = fdp_req.post(uri, "object", data=_object_data)['url']
-    except fdp_exc.RegistryAPICallError as e:
-        if not e.error_code == 409:
-            raise e
-        else:
-            raise fdp_exc.RegistryAPICallError(
-                f"Cannot post object"
-                f"'{_desc}', duplicate already exists",
-                error_code=409
-            )
-    except KeyError:
-        raise fdp_exc.InternalError(
-            f"Expected key 'url' in local registry API response"
-            f" for post object '{_desc}'"
-        )
-
-    # Get the name of the entry
-    if 'external_object' in data:
-        _name = data['external_object']
-    elif 'data_product' in data:
-        _name = data['data_product']
-    else:
-        raise fdp_exc.UserConfigError(
-            f"Failed to determine type while storing item '{local_file}'"
-            "into registry"
-        )
-
-    if 'data_product' in data['use']:
-        _name = data['use']['data_product']
-
-    _data_prod_data = {
-        "namespace": _namespace_url,
-        "object": _obj_url,
-        "version": str(data['use']['version']),
-        "name": _name
-    }
-
-    try:
-        _data_prod_url = fdp_req.post(uri, 'data_product', data=_data_prod_data)['url']
-    except fdp_exc.RegistryAPICallError as e:
-        if not e.error_code == 409:
-            raise e
-        else:
-            raise fdp_exc.RegistryAPICallError(
-                f"Cannot post data_product "
-                f"'{_name}', duplicate already exists",
-                error_code=409
-            )
-    except KeyError:
-        raise fdp_exc.InternalError(
-            f"Expected key 'url' in local registry API response"
-            f" for post object '{_name}'"
-        )
+    _data_prod_url = _get_url_from_data_product(
+        data=data,
+        label=local_file,
+        registry_uri=uri,
+        registry_token=token,
+        namespace_url=_namespace_url,
+        object_url=_obj_url,
+    )
 
     # If 'data_product' key present finish here and return URL
     # else this is an external object
-    if 'data_product' in data:
+    if "data_product" in data:
         return _data_prod_url
 
-    _expected_ext_obj_keys = [
-        "release_date",
-        "primary",
-        "title"
-    ]
+    return _get_url_from_external_obj(
+        data=data,
+        local_file=local_file,
+        registry_uri=uri,
+        registry_token=token,
+        data_product_url=_data_prod_url,
+    )
 
-    _identifier = None
-    _alternate_identifier = None
-    _alternate_identifier_type = None
 
-    if 'identifier' in data:
-        _identifier = data['identifier']
-        if not fdp_id.check_id_permitted(_identifier):
-            raise fdp_exc.UserConfigError(
-                f"Identifier '{_identifier}' is not a valid identifier"
-            )
+def _get_url_from_storage_loc(
+    local_file: str,
+    registry_uri: str,
+    registry_token: str,
+    relative_path: str,
+    root_store_url: str,
+    is_public: bool,
+) -> str:
+    _hash = calculate_file_hash(local_file)
 
-    if not _identifier:
-        if 'unique_name' not in data:
-            raise fdp_exc.UserConfigError(
-                "No identifier/alternate_identifier given for "
-                f"item '{local_file}'",
-                hint="You must provide either a URL 'identifier', or "
-                "'unique_name' and 'source_name' keys"
-            )
-        else:
-            _alternate_identifier = data['unique_name']
-            if 'alternate_identifier_type' in data:
-                _alternate_identifier_type = data['alternate_identifier_type']
-            else:
-                _alternate_identifier_type = 'local source descriptor'
+    _storage_loc_data = {
+        "path": relative_path,
+        "storage_root": root_store_url,
+        "public": is_public,
+        "hash": _hash,
+    }
+
+    _search_data = {"hash": _hash}
+
+    return fdp_req.post_else_get(
+        registry_uri,
+        "storage_location",
+        registry_token,
+        data=_storage_loc_data,
+        params=_search_data,
+    )
+
+
+def _get_url_from_file_type(
+    data: typing.Dict, local_file: str, registry_uri: str, registry_token: str
+) -> str:
+    if "file_type" in data:
+        _file_type = data["file_type"]
+    else:
+        _file_type = os.path.splitext(local_file)[1]
+
+    return create_file_type(registry_uri, _file_type, registry_token)
+
+
+def _get_url_from_namespace(
+    data: typing.Dict, label: str, registry_uri: str, registry_token: str
+) -> str:
+    # Namespace is read from the source information
+    if "namespace_name" not in data:
+        raise fdp_exc.UserConfigError(
+            f"Expected 'namespace_name' for item '{label}'"
+        )
+
+    _namespace_args = {
+        "name": data["namespace_name"],
+        "full_name": data["namespace_full_name"]
+        if "namespace_full_name" in data
+        else None,
+        "website": data.get("namespace_website", None),
+    }
+
+    return store_namespace(registry_uri, registry_token, **_namespace_args)
+
+
+def _get_url_from_external_obj(
+    data: typing.Dict,
+    local_file: str,
+    registry_uri: str,
+    registry_token: str,
+    data_product_url: str,
+) -> typing.Dict:
+    _expected_ext_obj_keys = ("release_date", "primary", "title")
 
     for key in _expected_ext_obj_keys:
         if key not in data:
@@ -591,19 +539,141 @@ def store_data_file(
             )
 
     _external_obj_data = {
-        "data_product": _data_prod_url,
-        "title": data['title'],
-        "primary_not_supplement": data['primary'],
-        "release_date": data['release_date'],
-        "identifier": _identifier,
-        "alternate_identifier": _alternate_identifier,
-        "alternate_identifier_type": _alternate_identifier_type
+        "data_product": data_product_url,
+        "title": data["title"],
+        "primary_not_supplement": data["primary"],
+        "release_date": data["release_date"],
+    }
+    _external_obj_data.update(_get_identifier_from_data(data, local_file))
+
+    return fdp_req.post(
+        registry_uri,
+        "external_object",
+        registry_token,
+        data=_external_obj_data,
+    )
+
+
+def _get_url_from_object(
+    data: typing.Dict,
+    registry_uri: str,
+    registry_token: str,
+    user: str,
+    storage_loc_url: str,
+    file_type_url: str,
+) -> str:
+    _desc = data.get("description", None)
+
+    _object_data = {
+        "description": _desc,
+        "file_type": file_type_url,
+        "storage_location": storage_loc_url,
+        "authors": [user],
     }
 
-    return fdp_req.post(uri, 'external_object', data=_external_obj_data)
+    try:
+        return fdp_req.post(
+            registry_uri, "object", registry_token, data=_object_data
+        )["url"]
+    except fdp_exc.RegistryAPICallError as e:
+        if e.error_code != 409:
+            raise e
+        else:
+            raise fdp_exc.RegistryAPICallError(
+                f"Cannot post object" f"'{_desc}', duplicate already exists",
+                error_code=409,
+            ) from e
+
+    except KeyError as e:
+        raise fdp_exc.InternalError(
+            f"Expected key 'url' in local registry API response"
+            f" for post object '{_desc}'"
+        ) from e
 
 
-def calculate_file_hash(file_name: str, buffer_size: int = 64*1024) -> str:
+def _get_url_from_data_product(
+    data: typing.Dict,
+    label: str,
+    registry_uri: str,
+    registry_token: str,
+    namespace_url: str,
+    object_url: str,
+) -> str:
+    """Retrieve the URL for a given config data product"""
+    # Get the name of the entry
+    if "external_object" in data:
+        _name = data["external_object"]
+    elif "data_product" in data:
+        _name = data["data_product"]
+    else:
+        raise fdp_exc.UserConfigError(
+            f"Failed to determine type while storing item '{label}'"
+            "into registry"
+        )
+
+    if "data_product" in data["use"]:
+        _name = data["use"]["data_product"]
+
+    _data_prod_data = {
+        "namespace": namespace_url,
+        "object": object_url,
+        "version": str(data["use"]["version"]),
+        "name": _name,
+    }
+
+    try:
+        return fdp_req.post(
+            registry_uri, "data_product", registry_token, data=_data_prod_data
+        )["url"]
+    except fdp_exc.RegistryAPICallError as e:
+        if e.error_code != 409:
+            raise e
+        else:
+            raise fdp_exc.RegistryAPICallError(
+                f"Cannot post data_product '{_name}', duplicate already exists",
+                error_code=409,
+            ) from e
+
+    except KeyError as e:
+        raise fdp_exc.InternalError(
+            f"Expected key 'url' in local registry API response"
+            f" for post object '{_name}'"
+        ) from e
+
+
+def _get_identifier_from_data(
+    data: typing.Dict, label: str
+) -> typing.Dict[str, str]:
+    """Retrieve the identifier metadata from the data entry"""
+    _identifier: typing.Dict[str, str] = {}
+
+    if data.get("identifier", None):
+        if not fdp_id.check_id_permitted(data["identifier"]):
+            raise fdp_exc.UserConfigError(
+                "Identifier '"
+                + data["identifier"]
+                + "' is not a valid identifier"
+            )
+        _identifier["identifier"] = data["identifier"]
+    else:
+        try:
+            _identifier["alternate_identifier"] = data["unique_name"]
+        except KeyError as e:
+            raise fdp_exc.UserConfigError(
+                "No identifier/alternate_identifier given for "
+                f"item '{label}'",
+                hint="You must provide either a URL 'identifier', or "
+                "'unique_name' and 'source_name' keys",
+            ) from e
+
+        _identifier["alternate_identifier"] = data.get(
+            "alternate_identifier_type", "local source descriptor"
+        )
+
+    return _identifier
+
+
+def calculate_file_hash(file_name: str, buffer_size: int = 64 * 1024) -> str:
     """Calculates the hash of a data file
 
     Parameters
@@ -619,21 +689,18 @@ def calculate_file_hash(file_name: str, buffer_size: int = 64*1024) -> str:
     # If the file is large we do not want to hash it in one go
     _input_hasher = hashlib.sha1()
 
-    with open(file_name, 'rb') as in_f:
+    with open(file_name, "rb") as in_f:
         _buffer = in_f.read(buffer_size)
         while len(_buffer) > 0:
             _input_hasher.update(_buffer)
             _buffer = in_f.read(buffer_size)
 
-    _hash = _input_hasher.hexdigest()
-
-    return _hash
+    return _input_hasher.hexdigest()
 
 
 def get_storage_root_obj_address(
-    remote_uri: str,
-    remote_token: str,
-    address_str: str) -> str:
+    remote_uri: str, remote_token: str, address_str: str
+) -> str:
     """Retrieve the RestAPI URL for a given storage location on the registry
 
     Parameters
@@ -653,19 +720,17 @@ def get_storage_root_obj_address(
     try:
         _results = fdp_req.get(
             remote_uri,
-            'storage_root',
-            params={
-                'root': address_str
-            },
-            token=remote_token
+            "storage_root",
+            params={"root": address_str},
+            token=remote_token,
         )
         if not _results:
             raise AssertionError
-    except (AssertionError, fdp_exc.RegistryAPICallError):
-        raise fdp_exc.RegistryAPICallError(
+    except (AssertionError, fdp_exc.RegistryAPICallError) as e:
+        raise fdp_exc.RegistryError(
             f"Cannot find a match for path '{address_str}' "
             f"from endpoint '{remote_uri}."
-        )
+        ) from e
 
 
 def check_match(input_object: str, results_list: typing.List[str]):
@@ -682,22 +747,23 @@ def check_match(input_object: str, results_list: typing.List[str]):
     results_list : typing.List[str]
         list of storage_location results matching search query
     """
-    _hash_list = [res['hash'] for res in results_list]
+    _hash_list = [res["hash"] for res in results_list]
     return calculate_file_hash(input_object) in _hash_list
 
 
 def check_if_object_exists(
-    cfg: typing.Dict,
+    local_uri: str,
     file_loc: str,
     obj_type: str,
     search_data: typing.Dict,
-    token: str = None) -> str:
+    token: str,
+) -> str:
     """Checks if a data product is already present in the registry
 
     Parameters
     ----------
-    cfg : typing.Dict
-        config yaml
+    local_uri : str
+        local registry endpoint
     file_loc : str
         path of file on system
     obj_type : str
@@ -713,53 +779,52 @@ def check_if_object_exists(
         whether object is present with matching hash, present or absent
         if present but not hash matched return the latest version identifier
     """
-    _logger = logging.getLogger("FAIRDataPipeline.Run")
-
-    _local_uri = fdp_conf.registry_url("local", cfg)
-
-
     _version = None
-    if 'version' in search_data:
-        _version = search_data['version']
-        if '${{' in _version:
-            del search_data['version']
+
+    if "version" in search_data:
+        _version = search_data["version"]
+        if "${{" in _version:
+            del search_data["version"]
 
     # Obtain list of storage_locations for the given data_product
     _results = fdp_req.get(
-        _local_uri,
-        obj_type,
-        params=search_data,
-        token=token
+        local_uri, obj_type, params=search_data, token=token
     )
 
-    _logger.debug(search_data)
-    _logger.debug(_results)
-
     try:
-        fdp_ver.get_correct_version(cfg, _results, False, _version)
+        fdp_ver.get_correct_version(
+            version=_version, results_list=_results, free_write=True
+        )
     except fdp_exc.UserConfigError:
         return "absent"
 
     if not _results:
         return "absent"
 
-    if obj_type == 'external_object':
-        _results = [res['data_product'] for res in _results]
-        _results = [fdp_req.url_get(r) for r in _results]
+    if obj_type == "external_object":
+        _results = [res["data_product"] for res in _results]
+        _results = [fdp_req.url_get(r, token=token) for r in _results]
 
-    _object_urls = [res['object'] for res in _results]
+    _object_urls = [res["object"] for res in _results]
 
     _storage_urls = [
-        fdp_req.url_get(obj_url, token=token)['storage_location']
+        fdp_req.url_get(obj_url, token=token)["storage_location"]
         for obj_url in _object_urls
     ]
 
     _storage_objs = [
-        fdp_req.url_get(store_url, token=token)
-        for store_url in _storage_urls
+        fdp_req.url_get(store_url, token=token) for store_url in _storage_urls
     ]
 
-    if check_match(file_loc, _storage_objs):
-        return "hash_match"
-    else:
-        return _results
+    return "hash_match" if check_match(file_loc, _storage_objs) else _results
+
+def get_upload_url(
+    file_hash: str,
+    remote_uri: str = None,
+    remote_token: str = None,
+) -> str:
+    if not remote_uri: 
+        remote_uri = fdp_conf.get_remote_token()
+    if not remote_token:
+        remote_token = fdp_conf.get_remote_token()
+    return fdp_req.post_upload_url(remote_uri, remote_token, file_hash)
\ No newline at end of file
